import { ICacheLayer } from '../index';

export interface IMemoryLayerOptions<K, V, CK> {
  map?: Map<CK, V>;

  keyFn?(key: K): CK;
}

export default class MemoryLayer<K, V, CK = K> implements ICacheLayer<K, V> {
  public get map() {
    return this._map;
  }

  private _map = new Map<CK, V>();
  private _options: IMemoryLayerOptions<K, V, CK>;

  constructor(options: IMemoryLayerOptions<K, V, CK> = {}) {
    this._options = {
      ...options,
    };
    if (options.map) {
      this._map = options.map;
    }
  }

  public async batchGet(keys: readonly K[]): Promise<Array<V | Error>> {
    return keys.map(key => {
      const cacheKey: CK = this._options.keyFn?.(key) ?? key as unknown as CK;
      const value = this._map.get(cacheKey);
      if (!value) {
        return new Error(`No cache value of key: ${key}`);
      }
      return value;
    });
  }

  public async batchSet(map: Map<K, V>) {
    map.forEach((value, key) => {
      const cacheKey: CK = this._options.keyFn?.(key) ?? key as unknown as CK;
      this._map.set(cacheKey, value);
    });
  }
}
